@@ -1,6 +1,6 @@
 """
 ///////////////////////////////////////////////////////////////////////////////
-//        Copyright (c) 2012-2020 Oscar Riveros. all rights reserved.        //
+//        Copyright (c) 2012-2021 Oscar Riveros. all rights reserved.        //
 //                        oscar.riveros@peqnp.science                        //
 //                                                                           //
 //   without any restriction, Oscar Riveros reserved rights, patents and     //
@@ -58,9 +58,8 @@ class Entity:
             self.data = self.block
         if not self.deep:
             self.deep = [self.bits]
-        if self.key is None:
-            self.key = self.encoder.new_key()
-            self.encoder.mapping(self.key, self.block)
+        self.key = self.encoder.new_key()
+        self.encoder.mapping(self.key, self.block)
 
     def is_in(self, item):
         bits = self.encoder.int(size=len(item))
@@ -264,9 +263,7 @@ class Entity:
             self.encoder.add_constraint(self.constraint[:-2], self.constraint[-2], self.constraint[-1])
             del self.constraint[:]
             return True
-        if other > 0:
-            return self.__gt__(other - 1)
-        return True
+        return self.__gt__(other - 1)
 
     def __neg__(self):
         if self.is_mip:
@@ -282,7 +279,6 @@ class Entity:
             return abs(self.value)
         lst = [self, -self]
         bits = self.encoder.int(size=len(lst))
-        self.encoder.variables.append(bits)
         assert sum(self.encoder.zero.iff(bits[i], self.encoder.one) for i in range(len(lst))) == self.encoder.one
         return sum(self.encoder.zero.iff(bits[i], lst[i]) for i in range(len(lst)))
 
@@ -356,14 +352,9 @@ class Entity:
             import functools
             import operator
             if isinstance(other, Entity):
-                return self.iff(functools.reduce(operator.and_, [
-                    self.encoder.zero.iff(bit[j], 1) for j in
-                    range(self.encoder.bits)])[0], other)
+                return self.iff(functools.reduce(operator.and_, [self.encoder.zero.iff(bit[j], self.encoder.one) for j in range(self.encoder.bits)])[0], other)
             else:
-                return self.iff(functools.reduce(operator.and_, [
-                    self.encoder.zero.iff(bit[j], 1) for j in
-                    range(self.encoder.bits)])[0],
-                    self.encoder.create_constant(other))
+                return self.iff(functools.reduce(operator.and_, [self.encoder.zero.iff(bit[j], self.encoder.one) for j in range(self.encoder.bits)])[0], self.encoder.create_constant(other))
         if isinstance(other, Entity):
             output_block = self.encoder.bv_mux_gate(self.block, other.block, bit)
             entity = Entity(self.encoder, block=output_block)
@@ -371,7 +362,7 @@ class Entity:
             return entity
         else:
             output_block = self.encoder.bv_mux_gate(self.block, self.encoder.create_constant(other), bit)
-            entity = Entity(self.encoder, key=str(other), block=output_block)
+            entity = Entity(self.encoder, block=output_block)
             self.encoder.variables.append(entity)
             return entity
 
