@@ -1,6 +1,6 @@
 """
 ///////////////////////////////////////////////////////////////////////////////
-//        Copyright (c) 2012-2020 Oscar Riveros. all rights reserved.        //
+//        Copyright (c) 2012-2021 Oscar Riveros. all rights reserved.        //
 //                        oscar.riveros@peqnp.science                        //
 //                                                                           //
 //   without any restriction, Oscar Riveros reserved rights, patents and     //
@@ -86,15 +86,21 @@ class CSP:
     @property
     def zero(self):
         if self.__0 is None:
-            self.__0 = self.int(value=0)
-            self.constants[0] = self.__0.block
+            if 0 in self.constants:
+                self.__0 = self.int(block=self.constants[0])
+            else:
+                self.__0 = self.int(value=0)
+                self.constants[0] = self.__0.block
         return self.__0
 
     @property
     def one(self):
         if self.__1 is None:
-            self.__1 = self.int(value=1)
-            self.constants[1] = self.__1.block
+            if 1 in self.constants:
+                self.__1 = self.int(block=self.constants[1])
+            else:
+                self.__1 = self.int(value=1)
+                self.constants[1] = self.__1.block
         return self.__1
 
     def add_variable(self):
@@ -339,7 +345,6 @@ class CSP:
         npos = self.and_gate(il=[lhs_msb, -rhs_msb])
         return self.or_gate(il=[npos, sleq], ol=ol)
 
-
     def bv_eq_gate(self, lhs_il, rhs_il, ol=None):
         if ol is None:
             ol = self.add_variable()
@@ -520,14 +525,15 @@ class CSP:
     def subset(self, k, data, empty=None, complement=False):
         x = self.int(size=len(data))
         self.at_most_k(x, k)
-        y = self.array(len(data))
+        y = self.array(dimension=len(data))
         if complement:
-            z = self.array(len(data))
+            z = self.array(dimension=len(data))
         for i in range(len(data)):
             assert self.zero.iff(x[i], data[i]) == self.zero.iff(x[i], y[i])
+            assert self.zero.iff(-x[i], self.zero if empty is None else empty) == self.zero.iff(-x[i], y[i])
             if complement:
                 assert self.zero.iff(-x[i], data[i]) == self.zero.iff(-x[i], z[i])
-            assert self.zero.iff(-x[i], self.zero if empty is None else empty) == self.zero.iff(-x[i], y[i])
+                assert self.zero.iff(x[i], self.zero if empty is None else empty) == self.zero.iff(x[i], z[i])
         if complement:
             return y, z
         return y
